@@ -4,9 +4,9 @@ use 5.004;
 use strict;
 use vars qw( $VERSION @EXPORT @EXPORT_OK $CurrentPackage @IncludeLibs );
 
-$VERSION   = '0.73';
+$VERSION   = '0.76';
 @EXPORT    = qw( scan_deps scan_deps_runtime );
-@EXPORT_OK = qw( scan_line scan_chunk add_deps scan_deps_runtime );
+@EXPORT_OK = qw( scan_line scan_chunk add_deps scan_deps_runtime path_to_inc_name );
 
 use Config;
 use Exporter;
@@ -25,6 +25,7 @@ use File::Temp ();
 use File::Spec ();
 use File::Basename ();
 use FileHandle;
+use Module::Build::ModuleInfo;
 
 =head1 NAME
 
@@ -32,8 +33,8 @@ Module::ScanDeps - Recursively scan Perl code for dependencies
 
 =head1 VERSION
 
-This document describes version 0.71 of Module::ScanDeps, released
-January  4, 2007.
+This document describes version 0.76 of Module::ScanDeps, released
+July 21, 2007.
 
 =head1 SYNOPSIS
 
@@ -121,7 +122,7 @@ If C<$execute> is an array reference, runs the files contained
 in it instead of C<@files>.
 
 Additionally, an option C<warn_missing> is recognized. If set to true,
-C<scan_deps> issues a warning to STDOUT for every module file that the
+C<scan_deps> issues a warning to STDERR for every module file that the
 scanned code depends but that wasn't found. Please note that this may
 also report numerous false positives. That is why by default, the heuristic
 silently drops all dependencies it cannot find.
@@ -165,6 +166,17 @@ modules that cannot be found are skipped.
 This function populates the C<%rv> hash with module/filename pairs, and
 returns a reference to it.
 
+=head2 B<path_to_inc_name>
+
+    $perl_name = path_to_inc_name($path, $warn)
+
+Assumes C<$path> refers to a perl file and does it's best to return the
+name as it would appear in %INC. Returns undef if no match was found 
+and a prints a warning to STDERR if C<$warn> is true.
+
+E.g. if C<$path> = perl/site/lib/Module/ScanDeps.pm then C<$perl_name>
+will be Module/ScanDeps.pm.
+
 =head1 NOTES
 
 You can set the global variable C<@Module::ScanDeps::IncludeLibs> to
@@ -307,15 +319,15 @@ my %Preload;
         _glob_in_inc('PDF/API2/Basic/TTF', 1);
     },
     'PDF/Writer.pm'                 => 'sub',
-    'POE'                           => [ qw(
+    'POE.pm'                           => [ qw(
         POE/Kernel.pm POE/Session.pm
     ) ],
-    'POE/Kernel.pm'                    => [
-        map "POE/Resource/$_.pm", qw(
-            Aliases Events Extrefs FileHandles
-            SIDs Sessions Signals Statistics
-        )
-    ],
+    'POE/Kernel.pm'                    => sub {
+	_glob_in_inc('POE/XS/Resource', 1),
+	_glob_in_inc('POE/Resource', 1),
+	_glob_in_inc('POE/XS/Loop', 1),
+	_glob_in_inc('POE/Loop', 1),
+    },
     'Parse/AFP.pm'                  => 'sub',
     'Parse/Binary.pm'               => 'sub',
     'PerlIO.pm'                     => [ 'PerlIO/scalar.pm' ],
@@ -405,6 +417,9 @@ my %Preload;
 
         return 'pod/perldiag.pod';
     },
+    'threads/shared.pm' => [qw( attributes.pm )],
+    	# anybody using threads::shared is likely to declare variables
+	# with attribute :shared
     'utf8.pm' => [
         'utf8_heavy.pl', do {
             my $dir = 'unicore';
@@ -438,6 +453,30 @@ my %Preload;
 
 # }}}
 
+sub path_to_inc_name($$) {
+    my $path = shift;
+    my $warn = shift;
+    my $inc_name;
+
+    if ($path =~ m/\.pm$/io) {
+        die "$path doesn't exist" unless (-f $path);
+        my $module_info = Module::Build::ModuleInfo->new_from_file($path);
+        die "Module::Build::ModuleInfo error: $!" unless defined($module_info);
+        $inc_name = $module_info->name();
+        if (defined($inc_name)) {
+            $inc_name =~ s|\:\:|\/|og;
+            $inc_name .= '.pm';
+        } else {
+            warn "# Couldn't find include name for $path\n" if $warn;
+        }
+    } else {
+        # Bad solution!
+        (my $vol, my $dir, $inc_name) = File::Spec->splitpath($path);
+    }
+
+    return $inc_name;
+}
+
 my $Keys = 'files|keys|recurse|rv|skip|first|execute|compile|warn_missing';
 sub scan_deps {
     my %args = (
@@ -445,6 +484,36 @@ sub scan_deps {
         (@_ and $_[0] =~ /^(?:$Keys)$/o) ? @_ : (files => [@_], recurse => 1)
     );
 
+    if (!defined($args{keys})) { 
+        $args{keys} = [map {path_to_inc_name($_, $args{warn_missing})} @{$args{files}}];
+    }
+
+    my ($type, $path);
+    foreach my $input_file (@{$args{files}}) {
+        $type = 'module';
+        $type = 'data' unless $input_file =~ /\.p[mh]$/io;
+        $path = $input_file;
+        if ($type eq 'module') {
+            # necessary because add_deps does the search for shared libraries and such
+            add_deps(
+                used_by => undef,
+                rv => $args{rv},
+                modules => [path_to_inc_name($path, $args{warn_missing})],
+                skip => undef,
+                warn_missing => $args{warn_missing},
+            );
+        }
+        else {
+            _add_info(
+                rv      => $args{rv},
+                module  => path_to_inc_name($path, $args{warn_missing}),
+                file    => $path,
+                used_by => undef,
+                type    => $type,
+            );
+        }
+    }
+
     scan_deps_static(\%args);
 
     if ($args{execute} or $args{compile}) {
@@ -485,13 +554,11 @@ sub scan_deps_static {
             foreach my $pm (scan_line($line)) {
                 last LINE if $pm eq '__END__';
                 
-                # Skip Tk hits from Term::ReadLine
+                # Skip Tk hits from Term::ReadLine and Tcl::Tk
                 my $pathsep = qr/\/|\\|::/;
-                if (
-                    $file =~ /${pathsep}Term${pathsep}ReadLine.pm$/
-                    and $pm =~ /^Tk\b/
-                ) {
-                    next;
+                if ($pm =~ /^Tk\b/) {
+                  next if $file =~ /(?:^|${pathsep})Term${pathsep}ReadLine\.pm$/;
+                  next if $file =~ /(?:^|${pathsep})Tcl${pathsep}Tk\W/;
                 }
 
                 if ($pm eq '__POD__') {
@@ -499,13 +566,13 @@ sub scan_deps_static {
                     next LINE;
                 }
 
-                $pm = 'CGI/Apache.pm' if /^Apache(?:\.pm)$/;
+                $pm = 'CGI/Apache.pm' if $file =~ /^Apache(?:\.pm)$/;
 
                 add_deps(
                     used_by => $key,
-                    rv      => $rv,
+                    rv      => $args->{rv},
                     modules => [$pm],
-                    skip    => $skip,
+                    skip    => $args->{skip},
                     warn_missing => $args->{warn_missing},
                 );
 
@@ -513,9 +580,9 @@ sub scan_deps_static {
 
                 add_deps(
                     used_by => $key,
-                    rv      => $rv,
+                    rv      => $args->{rv},
                     modules => $preload,
-                    skip    => $skip,
+                    skip    => $args->{skip},
                     warn_missing => $args->{warn_missing},
                 );
             }
@@ -615,8 +682,7 @@ sub scan_line {
 
         if (my ($libs) = /\b(?:use\s+lib\s+|(?:unshift|push)\W+\@INC\W+)(.+)/)
         {
-            my $archname =
-              defined($Config{archname}) ? $Config{archname} : '';
+            my $archname = defined($Config{archname}) ? $Config{archname} : '';
             my $ver = defined($Config{version}) ? $Config{version} : '';
             foreach (grep(/\w/, split(/["';() ]/, $libs))) {
                 unshift(@INC, "$_/$ver")           if -d "$_/$ver";
@@ -682,8 +748,8 @@ sub scan_chunk {
         }
         return $1 if /(?:^|\s)(?:do|require)\s+[^"]*"(.*?)"/;
         return $1 if /(?:^|\s)(?:do|require)\s+[^']*'(.*?)'/;
-        return $1 if /[^\$]\b([\w:]+)->\w/ and $1 ne 'Tk';
-        return $1 if /\b(\w[\w:]*)::\w+\(/;
+        return $1 if /[^\$]\b([\w:]+)->\w/ and $1 ne 'Tk' and $1 ne 'shift';
+        return $1 if /\b(\w[\w:]*)::\w+\(/ and $1 ne 'main' and $1 ne 'SUPER';
 
         if ($SeenTk) {
             my @modules;
@@ -725,9 +791,26 @@ sub _find_encoding {
 }
 
 sub _add_info {
-    my ($rv, $module, $file, $used_by, $type) = @_;
+    my %args = @_;
+    my ($rv, $module, $file, $used_by, $type) = @args{qw/rv module file used_by type/};
+
     return unless defined($module) and defined($file);
 
+    # Ensure file is always absolute
+    $file = File::Spec->rel2abs($file);
+    $file =~ s|\\|\/|go;
+
+    # Avoid duplicates that can arise due to case differences that don't actually 
+    # matter on a case tolerant system
+    if (File::Spec->case_tolerant()) {
+        foreach my $key (keys %$rv) {
+            if (lc($key) eq lc($module)) {
+                $module = $key;
+                last;
+            }
+        }
+    }
+
     $rv->{$module} ||= {
         file => $file,
         key  => $module,
@@ -737,40 +820,41 @@ sub _add_info {
     push @{ $rv->{$module}{used_by} }, $used_by
       if defined($used_by)
       and $used_by ne $module
-      and !grep { $_ eq $used_by } @{ $rv->{$module}{used_by} };
+      and ( (!File::Spec->case_tolerant() && !grep { $_ eq $used_by } @{ $rv->{$module}{used_by} })
+         or ( File::Spec->case_tolerant() && !grep { lc($_) eq lc($used_by) } @{ $rv->{$module}{used_by} }));
 }
 
+# This subroutine relies on not being called for modules that should be skipped
 sub add_deps {
     my %args =
       ((@_ and $_[0] =~ /^(?:modules|rv|used_by|warn_missing)$/)
         ? @_
         : (rv => (ref($_[0]) ? shift(@_) : undef), modules => [@_]));
 
-    my $rv   = $args{rv}   || {};
-    my $skip = $args{skip} || {};
+    my $rv = $args{rv}   || {};
     my $used_by = $args{used_by};
 
     foreach my $module (@{ $args{modules} }) {
+        my $file = _find_in_inc($module)
+          or _warn_of_missing_module($module, $args{warn_missing}), next;
+
         if (exists $rv->{$module}) {
-            _add_info($rv, undef, undef, $used_by, undef);
+            _add_info( rv     => $rv,      module  => $module,
+                       file   => $file,    used_by => $used_by,
+                       type   => undef );
             next;
         }
 
-        my $file = _find_in_inc($module)
-          or _warn_of_missing_module($module, $args{warn_missing}), next;
-        next if $skip->{$file};
-        next if is_insensitive_fs() and $skip->{lc($file)};
-
         my $type = 'module';
         $type = 'data' unless $file =~ /\.p[mh]$/i;
-        _add_info($rv, $module, $file, $used_by, $type);
+        _add_info( rv     => $rv,   module  => $module,
+                   file   => $file, used_by => $used_by,
+                   type   => $type );
 
         if ($module =~ /(.*?([^\/]*))\.p[mh]$/i) {
             my ($path, $basename) = ($1, $2);
 
             foreach (_glob_in_inc("auto/$path")) {
-                next if $skip->{$_->{file}};
-                next if is_insensitive_fs() and $skip->{lc($_->{file})};
                 next if $_->{file} =~ m{\bauto/$path/.*/};  # weed out subdirs
                 next if $_->{name} =~ m/(?:^|\/)\.(?:exists|packlist)$/;
                 my $ext = lc($1) if $_->{name} =~ /(\.[^.]+)$/;
@@ -779,8 +863,9 @@ sub add_deps {
                 $type = 'autoload' if $ext eq '.ix' or $ext eq '.al';
                 $type ||= 'data';
 
-                _add_info($rv, "auto/$path/$_->{name}", $_->{file}, $module,
-                    $type);
+                _add_info( rv     => $rv,        module  => "auto/$path/$_->{name}",
+                           file   => $_->{file}, used_by => $module,
+                           type   => $type );
             }
         }
     }
@@ -872,7 +957,7 @@ sub calculate_info {
 
     my $info = {
         main => {  file     => $self->{main}{file},
-            store_as => $self->{main}{key},
+                   store_as => $self->{main}{key},
         },
     };
 
@@ -890,6 +975,7 @@ sub calculate_info {
     foreach my $key (sort keys %{$rv}) {
         my $val = $rv->{$key};
         if ($cache{ $val->{key} }) {
+            defined($val->{used_by}) or next;
             push @{ $info->{ $val->{type} }->{ $val->{key} }->{used_by} },
               @{ $val->{used_by} };
         }
@@ -1084,7 +1170,12 @@ sub _merge_rv {
 
 sub _not_dup {
     my ($key, $rv1, $rv2) = @_;
-    (_abs_path($rv1->{$key}{file}) ne _abs_path($rv2->{$key}{file}));
+    if (File::Spec->case_tolerant()) {
+        return lc(_abs_path($rv1->{$key}{file})) ne lc(_abs_path($rv2->{$key}{file}));
+    }
+    else {
+        return _abs_path($rv1->{$key}{file}) ne _abs_path($rv2->{$key}{file});
+    }
 }
 
 sub _abs_path {
@@ -1101,7 +1192,7 @@ sub _warn_of_missing_module {
     my $warn = shift;
     return if not $warn;
     return if not $module =~ /\.p[ml]$/;
-    print "# Could not find source file '$module' in \@INC or \@IncludeLibs. Skipping it.\n"
+    warn "# Could not find source file '$module' in \@INC or \@IncludeLibs. Skipping it.\n"
       if not -f $module;
 }
 
